@@ -86,6 +86,14 @@ class FakeResource(dict):
         self._add_details(info)
 
 
+class FakeOpenStackSDKResource(FakeResource):
+    def __getattr__(self, attr):
+        if attr in self._info:
+            return self._info[attr]
+        else:
+            raise AttributeError("No such attribute '{}'".format(attr))
+
+
 class FakeOvsdbRow(FakeResource):
     """Fake one or more OVSDB rows."""
 
@@ -165,8 +173,8 @@ class FakeSubnet():
         # Overwrite default attributes.
         subnet_attrs.update(attrs)
 
-        return FakeResource(info=copy.deepcopy(subnet_attrs),
-                            loaded=True)
+        return FakeOpenStackSDKResource(info=copy.deepcopy(subnet_attrs),
+                                        loaded=True)
 
 
 class FakeOVNPort():
@@ -245,6 +253,28 @@ class FakeOVNRouter():
         return type('Logical_Router', (object, ), router_attrs)
 
 
+class FakeOVNLB():
+
+    @staticmethod
+    def create_one_lb(attrs=None):
+        fake_uuid = uuidutils.generate_uuid()
+        lb_attrs = {
+            'uuid': fake_uuid,
+            'external_ids': {},
+            'health_check': [],
+            'ip_port_mappings': {},
+            'name': '',
+            'options': {},
+            'protocol': 'tcp',
+            'selection_fields': [],
+            'vips': {}
+        }
+
+        # Overwrite default attributes.
+        lb_attrs.update(attrs)
+        return type('Load_Balancer', (object, ), lb_attrs)
+
+
 class FakePort():
     """Fake one or more ports."""
 
@@ -289,8 +319,8 @@ class FakePort():
         # Overwrite default attributes.
         port_attrs.update(attrs)
 
-        return FakeResource(info=copy.deepcopy(port_attrs),
-                            loaded=True)
+        return FakeOpenStackSDKResource(info=copy.deepcopy(port_attrs),
+                                        loaded=True)
 
 
 class FakeLB(data_models.LoadBalancer):
